local Cryolysis2 = Cryolysis2
local L = AceLibrary("AceLocale-2.2"):new("Cryolysis2")
local BS = AceLibrary("Babble-Spell-2.2")
local BZ = AceLibrary("Babble-Zone-2.2")

function Cryolysis2:CreateOptions()
	-- Create array of slash commands
	Cryolysis2.slashopts = {
		type = "group",
		args = {
			menu = {
				type = "execute",
				name = L["Show Menu"],
				desc = L["Open the configuration menu."],
				func = function() Cryolysis2:OpenMenu() end
			}
		}
	}

	-- Create array of options
	Cryolysis2.options = {
		name = "Cryolysis 2",
		type = 'group',
		args = {
			gen = {
				type = "group",
				name = L["General Settings"],
				desc = L["Adjust various settings for Cryolysis 2."],
				order = 5,
				args = {
					lock = {
						type = "toggle",
						name = L["Lock Sphere and Buttons"],
						desc = L["Lock the main sphere and buttons so they can't be moved."],
						get = function() return Cryolysis2.db.profile.LockSphere end,
						set = function(v) Cryolysis2.db.profile.LockSphere = v end,
						order = 1
					},
					constrict = {
						type = "toggle",
						name = L["Constrict Buttons to Sphere"],
						desc = L["Lock the buttons in place around the main sphere."],
						get = function() return Cryolysis2.db.profile.LockButtons end,
						set = function(v) 
						Cryolysis2.db.profile.LockButtons = v 
						Cryolysis2:UpdateAllButtonPositions()
						end,
						order = 2
					},
					tooltips = {
						type = "toggle",
						name = L["Hide Tooltips"],
						desc = L["Hide the main sphere and button tooltips."],
						get = function() return Cryolysis2.db.profile.HideTooltips end,
						set = function(v) Cryolysis2.db.profile.HideTooltips = v end,
						order = 2.8
					}
				}
			},
			autorestock = {
				type = "group",
				name = L["Reagent Restocking"],
				desc = L["Adjust various settings for the automatic reagent restocking feature."],
				order = 10,
				args = {
					reagents = {
						type = "toggle",
						name = L["Restock Reagents"],
						desc = L["Automatically restock reagents when talking to a reagent vendor."],
						get = function() return Cryolysis2.db.profile.Restock end,
						set = function(v) Cryolysis2.db.profile.Restock = v end,
						order = 1
					},
					restockconfirm = {
						type = "toggle",
						name = L["Confirm Restocking"],
						desc = L["Pop-up a confirmation box before restocking reagents."],
						get = function() return Cryolysis2.db.profile.RestockConfirm end,
						set = function(v) Cryolysis2.db.profile.RestockConfirm = v end,
						order = 2
					},
					apamount = {
						type = "range",
						name = select(1,GetItemInfo("Arcane Powder")),
						desc = L["Adjust the amount of Arcane Powder to restock to."],
						get = function() return Cryolysis2.db.profile.RestockPowder end,
						set = function(v) Cryolysis2.db.profile.RestockPowder = v end,
						min = 0,
						max = 100,
						step = 1,
						order = 3
					},
					teleamount = {
						type = "range",
						name = select(1,GetItemInfo("Rune of Teleportation")),
						desc = L["Adjust the amount of Runes of Teleportation to restock to."],
						get = function() return Cryolysis2.db.profile.RestockTeleport end,
						set = function(v) Cryolysis2.db.profile.RestockTeleport = v end,
						min = 0,
						max = 100,
						step = 1,
						order = 4
					},
					portamount = {
						type = "range",
						name = select(1,GetItemInfo("Rune of Portals")),
						desc = L["Adjust the amount of Runes of Portals to restock to."],
						get = function() return Cryolysis2.db.profile.RestockPortals end,
						set = function(v) Cryolysis2.db.profile.RestockPortals = v end,
						min = 0,
						max = 100,
						step = 1,
						order = 5
					}
				}
			},
			timersettings = {
				type = "group",
				name = L["Timer Settings"],
				desc = L["Adjust various settings for the spell timers."],
				order = 15,
				args = {
					timertoggle = {
						type = "toggle",
						name = L["Timers On/Off"],
						desc = L["Turn timers on or off."],
						get = function() return Cryolysis2.db.profile.ShowSpellTimers end,
						set = function(v) 
						Cryolysis2.db.profile.ShowSpellTimers = v 
						Cryolysis2:UpdateHideShowSpellTimers()
						end,
						order = 16
					},
					timerlock = {
						type = "toggle",
						name = L["Lock Timer Frame"],
						desc = L["Lock the timer frame in place so it can't move."],
						get = function() return Cryolysis2.db.profile.LockTimers end,
						set = function(v) 
						Cryolysis2.db.profile.LockTimers = v end,
						order = 16.5
					},
					cooldown = {
						type = "group",
						name = L["Cooldown Timers"],
						desc = L["Select which spells have their cooldowns shown in the timer window."],
						order = 17,
						args = {
                            cooldowntoggle = {
                                type = "toggle",
                                name = L["Cooldown Timers On/Off"],
                                desc = L["Turn timers on or off."],
                                get = function() return Cryolysis2.db.profile.ShowCooldownTimers end,
                                set = function(v) 
                                    Cryolysis2.db.profile.ShowCooldownTimers = v 
                                    Cryolysis2:UpdateHideShowSpellTimers()
                                    end,
                                order = 16
                            }
						}
					},
					debuff = {
						type = "group",
						name = L["Debuff Timers"],
						desc = L["Select which spells have their debuffs tracked in the timer window."],
						order = 18,
						args = {
--~                             debufftoggle = {
--~                                 type = "toggle",
--~                                 name = L["Debuff Timers On/Off"],
--~                                 desc = L["Turn timers on or off."],
--~                                 get = function() return Cryolysis2.db.profile.ShowDebuffTimers end,
--~                                 set = function(v) 
--~                                     Cryolysis2.db.profile.ShowDebuffTimers = v 
--~                                     Cryolysis2:UpdateHideShowSpellTimers()
--~                                     end,
--~                                 order = 16
--~                             }
						}
					}
				}
			},
			messagesettings = {
				type = "group",
				name = L["Message Settings"],
				desc = L["Adjust various settings for teleport, portal, and polymorph messages."],
				order = 17,
				args = {
					chattype = {
						type = "text",
						name = L["Chat Channel"],
						desc = L["Choose which chat channel messages are displayed in."],
						get = function() return Cryolysis2.db.profile.ChatChannel end,
						set = function(v) Cryolysis2.db.profile.ChatChannel = v end,
						order = 2,
						validate = {
							["USER"]	= L["User"],
							["SAY"]		= L["Say"],
							["PARTY"]	= L["Party"],
							["RAID"]	= L["Raid"],
							["GROUP"]	= L["Group"],
							["WORLD"]	= L["World"]
						}
							
					},
					sm = {
						type = "toggle",
						name = L["Short Messages"],
						desc = L["Display shortened versions of polymorph, teleport, portal, and mount messages."],
						get = function() return Cryolysis2.db.profile.SM end,
						set = function(v) Cryolysis2.db.profile.SM = v end,
						order = 5
					},
					poly = {
						type = "toggle",
						name = L["Polymorph Messages"],
						desc = L["Display a message in chat when casting polymorph."],
						get = function() return Cryolysis2.db.profile.PolyMessage end,
						set = function(v) Cryolysis2.db.profile.PolyMessage = v end,
						order = 10
					},
					polywarn = {
						type = "toggle",
						name = L["Polymorph Warning"],
						desc = L["Play a sound when polymorph is about to break."],
						get = function() return Cryolysis2.db.profile.PolyWarn end,
						set = function(v) Cryolysis2.db.profile.PolyWarn = v end,
						order = 11
					},
					polywarntime = {
						type = "range",
						name = L["Polymorph Warning Time"],
						desc = L["Set how long before polymorph breaks to play the sound."],
						get = function() return Cryolysis2.db.profile.PolyWarnTime end,
						set = function(v) Cryolysis2.db.profile.PolyWarnTime = v end,
						min = 1,
						max = 10,
						step = 1,
						order = 12
					},
					polybreak = {
						type = "toggle",
						name = L["Polymorph Break"],
						desc = L["Play a sound when polymorph breaks."],
						get = function() return Cryolysis2.db.profile.PolyBreak end,
						set = function(v) Cryolysis2.db.profile.PolyBreak = v end,
						order = 13
					},
					ritual = {
						type = "toggle",
						name = L["Ritual Message"],
						desc = L["Display a message in chat when casting Ritual of Refreshment."],
						get = function() return Cryolysis2.db.profile.RitualMessage end,
						set = function(v) Cryolysis2.db.profile.RitualMessage = v end,
						order = 14
					},
					mount = {
						type = "toggle",
						name = L["Mount Messages"],
						desc = L["Display a message in chat when mounting."],
						get = function() return Cryolysis2.db.profile.MountMessage end,
						set = function(v) Cryolysis2.db.profile.MountMessage = v end,
						order = 15
					},
					tele = {
						type = "toggle",
						name = L["Teleport Messages"],
						desc = L["Display a message in chat when teleporting."],
						get = function() return Cryolysis2.db.profile.TeleMessage end,
						set = function(v) Cryolysis2.db.profile.TeleMessage = v end,
						order = 20
					},
					port = {
						type = "toggle",
						name = L["Portal Messages"],
						desc = L["Display a message in chat when opening a portal."],
						get = function() return Cryolysis2.db.profile.PortalMessage end,
						set = function(v) Cryolysis2.db.profile.PortalMessage = v end,
						order = 25
					}
				}
			},
			graphicalsettings = {
				type = "group",
				name = L["Button Settings"],
				desc = L["Adjust various settings for each button."],
				order = 20,
				args = {
					msbutton = {
						type = "group",
						name = L["Main Sphere"],
						desc = L["Adjust various settings for the main sphere."],
						order = 20.5,
						args = {
							mshide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the main sphere."],
								get = function() return Cryolysis2.db.profile.MSHide end,
								set = function(v)
								Cryolysis2.db.profile.MSHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							insphere = {
								type = "text",
								name = L["Text"],
								desc = L["Adjust what information is displayed on the sphere."],
								get = function() return Cryolysis2.db.profile.InnerSphere end,
								set = function(v)
									Cryolysis2.db.profile.InnerSphere = v 
									Cryolysis2:UpdateInnerSphere() 
								end,
								order = 2,
								validate = {
									["1"]	= L["Nothing"],
									["2"]	= L["Food/Drink"],
									["3"]	= L["Drink/Food"],
									["4"]	= L["Current Health"],
									["5"]	= L["Health %"],
									["6"]	= L["Current Mana"],
									["7"]	= L["Mana %"],
									["8"]	= L["Mana Gem Cooldown"],
									["9"]	= L["Evocation Cooldown"]
								}
							},
							spheremb = {
								type = "text",
								name = L["Inner Sphere"],
								desc = L["Adjust what clicking the inner sphere does."],
								get = function() return Cryolysis2.db.profile.MainButton end,
								set = function(v) 
									Cryolysis2.db.profile.MainButton = v
									Cryolysis2:UpdateSphereAttributes()
								end,
								order = 3,
								validate = {
									["1"]	= L["Nothing"],
									["2"]	= L["Eat and drink"],
									["3"]	= L["Evocation"],
									["4"]	= L["Polymorph"],
									["5"]	= L["Use Mana Gem"],
									["6"]	= L["Blink"]
								}
							},
							outsphere = {
								type = "text",
								name = L["Outer Sphere"],
								desc = L["Adjust what information is displayed using the outer sphere."],
								get = function() return Cryolysis2.db.profile.OuterSphere end,
								set = function(v) 
									Cryolysis2.db.profile.OuterSphere = v 
									Cryolysis2:UpdateOuterSphere()
								end,
								order = 4,
								validate = {
									["1"]	= L["Nothing"],
									["2"]	= L["Health"],
									["3"]	= L["Mana"],
									["4"]	= L["Mana Gem Cooldown"],
									["5"]	= L["Evocation Cooldown"]
								}
							},
							angleadj = {
								type = "range",
								name = L["Angle"],
								desc = L["Adjust the position of the buttons around the sphere."],
								get = function() return Cryolysis2.db.profile.Angle end,
								set = function(v) 
								Cryolysis2.db.profile.Angle = v
								Cryolysis2:UpdateAllButtonPositions()
								end,
								min = 0,
								max = 360,
								step = 18,
								order = 5
							},
							scalems = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the main sphere."],
								get = function() return Cryolysis2.db.profile.MainSphereScale end,
								set = function(v) 
								Cryolysis2.db.profile.MainSphereScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true,
								order = 6
							},
							scalebuttons = {
								type = "range",
								name = L["Scale All Buttons"],
								desc = L["Scale the size of all of the buttons at once."],
								get = function() return Cryolysis2.db.profile.ButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.ButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true,
								order = 7
							}
						}
					},
					foodbutton = {
						type = "group",
						name = L["Food Button"],
						desc = L["Adjust various settings for the food button."],
						order = 21,
						args = {
							foodhide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.FoodHide end,
								set = function(v)
								Cryolysis2.db.profile.FoodHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							foodcount = {
								type = "toggle",
								name = L["Show Food Count"],
								desc = L["Show how much conjured food you have on the food button."],
								get = function() return Cryolysis2.db.profile.ShowFoodCountText end,
								set = function(v) Cryolysis2.db.profile.ShowFoodCountText = v end
							},
							scalefood = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the food button."],
								get = function() return Cryolysis2.db.profile.FoodButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.FoodButtonScale = v
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							movefood = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2FoodButton") end,
							}
						}
					},
					waterbutton = {
						type = "group",
						name = L["Water Button"],
						desc = L["Adjust various settings for the water button."],
						order = 22,
						args = {
							waterhide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.WaterHide end,
								set = function(v)
								Cryolysis2.db.profile.WaterHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							watercount = {
								type = "toggle",
								name = L["Show Water Count"],
								desc = L["Show how much conjured water you have on the water button."],
								get = function() return Cryolysis2.db.profile.ShowWaterCountText end,
								set = function(v) Cryolysis2.db.profile.ShowWaterCountText = v end
							},
							scalewater = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the water button."],
								get = function() return Cryolysis2.db.profile.WaterButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.WaterButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							movewater = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2WaterButton") end,
							}
						}
					},
					gembutton = {
						type = "group",
						name = L["Mana Gem Button"],
						desc = L["Adjust various settings for the mana gem button."],
						order = 23,
						args = {
							gemhide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.GemHide end,
								set = function(v)
								Cryolysis2.db.profile.GemHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							portgrowleft = {
								type = "toggle",
								name = L["Grow Left"],
								desc = L["Make the menu grow left instead right."],
								get = function() return Cryolysis2.db.profile.GemMenuGrowLeft end,
								set = function(v) 
								Cryolysis2.db.profile.GemMenuGrowLeft = v
								Cryolysis2:UpdateManaGemButton()
								end,
							},
							gem = {
								type = "toggle",
								name = L["Show Cooldown"],
								desc = L["Show the mana gem cooldown on the mana gem button."],
								get = function() return Cryolysis2.db.profile.ShowManaGemCooldownText end,
								set = function(v) 
								Cryolysis2.db.profile.ShowManaGemCooldownText = v
								Cryolysis2:Cryo2_UpdateManaGemCooldown() 
								end,
								order = 7
							},
							scalegem = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the mana gem button."],
								get = function() return Cryolysis2.db.profile.ManaGemButtonScale end,
								set = function(v)
								Cryolysis2.db.profile.ManaGemButtonScale = v
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							movegem = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2ManaGemButton") end,
							},
							menuofsx = {
								type = "range",
								name = L["Menu X Offset"],
								desc = L["Adjust the left-to-right position of the menu."],
								get = function() return Cryolysis2.db.profile.GemMenuOfsX end,
								set = function(v) 
								Cryolysis2.db.profile.GemMenuOfsX = v
								Cryolysis2:UpdateManaGemButton()
								end,
								min = -100,
								max = 100,
								step = 1
							},
							menuofsy = {
								type = "range",
								name = L["Menu Y Offset"],
								desc = L["Adjust the left-to-right position of the menu."],
								get = function() return Cryolysis2.db.profile.GemMenuOfsY end,
								set = function(v) 
								Cryolysis2.db.profile.GemMenuOfsY = v 
								Cryolysis2:UpdateManaGemButton()
								end,
								min = -100,
								max = 100,
								step = 1
							}
						}
					},
					leftcustom = {
						type = "group",
						name = L["Left Custom Button"],
						desc = L["Adjust various settings for the left custom button."],
						order = 24,
						args = {
							lefthide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.LeftHide end,
								set = function(v)
								Cryolysis2.db.profile.LeftHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							scaleleft = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the left custom button."],
								get = function() return Cryolysis2.db.profile.LeftCustomButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.LeftCustomButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
								order = 6,
                                step = .1,
								isPercent = true
							},
							buttontype = {
								type = "text",
								name = L["Button Type"],
								desc = L["Choose whether this button casts a spell, uses a macro, or uses an item."],
								get = function() return Cryolysis2.db.profile.LeftCustomButtonType end,
								set = function(v) 
								Cryolysis2.db.profile.LeftCustomButtonType = v
								Cryolysis2:UpdateCustomButtonAttributes("left")
								end,
								order = 2,
								validate = {["spell"] = L["Spell"], ["macro"] = L["Macro"], ["item"] = L["Item"]}
							},
							leftleftclick = {
								type = "text",
								name = L["Left Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.LeftSpellLeft end,
								set = function(v) 
								Cryolysis2.db.profile.LeftSpellLeft = v
								Cryolysis2:UpdateLeftButton(1) 
								end,
								order = 3
							},
							leftrightclick = {
								type = "text",
								name = L["Right Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.LeftSpellRight end,
								set = function(v) 
								Cryolysis2.db.profile.LeftSpellRight = v
								Cryolysis2:UpdateLeftButton(2) 
								end,
								order = 4
							},
							leftmidclick = {
								type = "text",
								name = L["Middle Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.LeftSpellMid end,
								set = function(v) 
								Cryolysis2.db.profile.LeftSpellMid = v 
								Cryolysis2:UpdateLeftButton(3) 
								end,
								order = 5
							},
							moveleft = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2LeftCustomButton") end,
								order = 7
							}
						}
					},
					evocationbutton = {
						type = "group",
						name = L["Evocation Button"],
						desc = L["Adjust various settings for the evocation button."],
						order = 24.5,
						args = {
							evochide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.EvocHide end,
								set = function(v)
								Cryolysis2.db.profile.EvocHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							evoc = {
								type = "toggle",
								name = L["Show Cooldown"],
								desc = L["Show evocation's cooldown on the evocation button."],
								get = function() return Cryolysis2.db.profile.ShowEvocationCooldownText end,
								set = function(v) 
								Cryolysis2.db.profile.ShowEvocationCooldownText = v
								Cryolysis2:Cryo2_UpdateEvocationCooldown()
								end,
								order = 6
							},
							scaleevoc = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the evocation button."],
								get = function() return Cryolysis2.db.profile.EvocationButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.EvocationButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							moveevoc = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2EvocButton") end,
							}
						}
					},
					rightcustom = {
						type = "group",
						name = L["Right Custom Button"],
						desc = L["Adjust various settings for the right custom button."],
						order = 25,
						args = {
							righthide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.RightHide end,
								set = function(v)
								Cryolysis2.db.profile.RightHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							scalerightcustom = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the right custom button."],
								get = function() return Cryolysis2.db.profile.RightCustomButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.RightCustomButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							buttontype = {
								type = "text",
								name = L["Button Type"],
								desc = L["Choose whether this button casts a spell, uses a macro, or uses an item."],
								get = function() return Cryolysis2.db.profile.RightCustomButtonType end,
								set = function(v) 
								Cryolysis2.db.profile.RightCustomButtonType = v
								Cryolysis2:UpdateCustomButtonAttributes("right")
								end,
								validate = {["spell"] = L["Spell"], ["macro"] = L["Macro"], ["item"] = L["Item"]}
							},
							rightleftclick = {
								type = "text",
								name = L["Left Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.RightSpellLeft end,
								set = function(v) 
								Cryolysis2.db.profile.RightSpellLeft = v
								Cryolysis2:UpdateRightButton(1) 
								end
							},
							rightrightclick = {
								type = "text",
								name = L["Right Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.RightSpellRight end,
								set = function(v) 
								Cryolysis2.db.profile.RightSpellRight = v
								Cryolysis2:UpdateRightButton(2) 
								end
							},
							rightmidclick = {
								type = "text",
								name = L["Middle Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.RightSpellMid end,
								set = function(v) 
								Cryolysis2.db.profile.RightSpellMid = v 
								Cryolysis2:UpdateRightButton(3) 
								end
							},
							moveright = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2RightCustomButton") end,
							}
						}
					},
					spellbutton = {
						type = "group",
						name = L["Buff Menu Button"],
						desc = L["Adjust various settings for the buff menu button."],
						order = 25.5,
						args = {
							buffhide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.BuffHide end,
								set = function(v)
								Cryolysis2.db.profile.BuffHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							buffgrowleft = {
								type = "toggle",
								name = L["Grow Left"],
								desc = L["Make the menu grow left instead right."],
								get = function() return Cryolysis2.db.profile.BuffMenuGrowLeft end,
								set = function(v) 
								Cryolysis2.db.profile.BuffMenuGrowLeft = v
								Cryolysis2:UpdateSpellMenu()
								end,
							},
							feathers = {
								type = "toggle",
								name = L["Light Feather Count"],
								desc = L["Show how many light feathers you have on the slow fall button."],
								get = function() return Cryolysis2.db.profile.ShowFeatherCountText end,
								set = function(v) 
								Cryolysis2.db.profile.ShowFeatherCountText = v 
								Cryolysis2:ShowHideFeatherCount()
								end
							},
							powder = {
								type = "toggle",
								name = L["Arcane Powder Count"],
								desc = L["Show how much arcane powder you have on the arcane intellect button."],
								get = function() return Cryolysis2.db.profile.ShowPowderCountText end,
								set = function(v)
								Cryolysis2.db.profile.ShowPowderCountText = v
								Cryolysis2:ShowHideArcanePowderCount()
								end
							},
							scalespell = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the spell menu button."],
								get = function() return Cryolysis2.db.profile.SpellButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.SpellButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							movebuff = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2BuffMenuButton") end,
							},
							menuofsx = {
								type = "range",
								name = L["Menu X Offset"],
								desc = L["Adjust the left-to-right position of the menu."],
								get = function() return Cryolysis2.db.profile.BuffMenuOfsX end,
								set = function(v) 
								Cryolysis2.db.profile.BuffMenuOfsX = v 
								Cryolysis2:UpdateSpellMenu()
								end,
								min = -100,
								max = 100,
								step = 1
							},
							menuofsy = {
								type = "range",
								name = L["Menu Y Offset"],
								desc = L["Adjust the left-to-right position of the menu."],
								get = function() return Cryolysis2.db.profile.BuffMenuOfsY end,
								set = function(v) 
								Cryolysis2.db.profile.BuffMenuOfsY = v 
								Cryolysis2:UpdateSpellMenu()
								end,
								min = -100,
								max = 100,
								step = 1
							}
						}
					},
					mountbutton = {
						type = "group",
						name = L["Mount Button"],
						desc = L["Adjust various settings for the mount button."],
						order = 26,
						args = {
							mounthide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.MountHide end,
								set = function(v)
								Cryolysis2.db.profile.MountHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							scalemount = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the mount button."],
								get = function() return Cryolysis2.db.profile.MountButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.MountButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							movemount = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2MountButton") end,
							},
                            leftmount = {
                                type = "text",
                                name = "Left-Click Mount",
                                desc = "Set which mount is used when left-clicking the mount button.",
                                get = function() return Cryolysis2.db.profile.LeftMount end,
                                set = function(v) 
                                    Cryolysis2.db.profile.LeftMount = v 
                                    Cryolysis2:UpdateMountButton()
                                    end,
                                validate = function() return Cryolysis2.Private.Mounts end,
                            },
                            rightmount = {
                                type = "text",
                                name = "Right-Click Mount",
                                desc = "Set which mount is used when right-clicking the mount button.",
                                get = function() return Cryolysis2.db.profile.RightMount end,
                                set = function(v) 
                                    Cryolysis2.db.profile.RightMount = v 
                                    Cryolysis2:UpdateMountButton()
                                    end,
                                validate = function() return Cryolysis2.Private.Mounts end,
                            }
						}
					},
					teleandportbutton = {
						type = "group",
						name = L["Teleport/Portal Menu Button"],
						desc = L["Adjust various settings for the teleport/portal menu button."],
						order = 27,
						args = {
							porthide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.PortHide end,
								set = function(v)
								Cryolysis2.db.profile.PortHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							portgrowleft = {
								type = "toggle",
								name = L["Grow Left"],
								desc = L["Make the menu grow left instead right."],
								get = function() return Cryolysis2.db.profile.PortMenuGrowLeft end,
								set = function(v) 
								Cryolysis2.db.profile.PortMenuGrowLeft = v
								Cryolysis2:UpdateTeleAndPortButton()
								end,
							},
							scaleteleandport = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the teleport/portal button."],
								get = function() return Cryolysis2.db.profile.TeleAndPortButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.TeleAndPortButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							moveteleandport = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2TeleAndPortMenuButton") end,
							},
							menuofsx = {
								type = "range",
								name = L["Menu X Offset"],
								desc = L["Adjust the left-to-right position of the menu."],
								get = function() return Cryolysis2.db.profile.PortMenuOfsX end,
								set = function(v) 
								Cryolysis2.db.profile.PortMenuOfsX = v 
								Cryolysis2:UpdateTeleAndPortButton()
								end,
								min = -100,
								max = 100,
								step = 1
							},
							menuofsy = {
								type = "range",
								name = L["Menu Y Offset"],
								desc = L["Adjust the left-to-right position of the menu."],
								get = function() return Cryolysis2.db.profile.PortMenuOfsY end,
								set = function(v) 
								Cryolysis2.db.profile.PortMenuOfsY = v 
								Cryolysis2:UpdateTeleAndPortButton()
								end,
								min = -100,
								max = 100,
								step = 1
							}
						}
					},
					bottomcustom = {
						type = "group",
						name = L["Bottom Custom Button"],
						desc = L["Adjust various settings for the bottom custom button."],
						order = 28,
						args = {
							bottomhide = {
								type = "toggle",
								name = L["Hide"],
								desc = L["Show or hide the button."],
								get = function() return Cryolysis2.db.profile.BottomHide end,
								set = function(v)
								Cryolysis2.db.profile.BottomHide = v
								Cryolysis2:UpdateHideShow()
								end,
								order = 1
							},
							scalebottomcustom = {
								type = "range",
								name = L["Scale"],
								desc = L["Scale the size of the bottom custom button."],
								get = function() return Cryolysis2.db.profile.RightCustomButtonScale end,
								set = function(v) 
								Cryolysis2.db.profile.BottomCustomButtonScale = v 
								Cryolysis2:UpdateAllButtonPositions()
								Cryolysis2:UpdateAllButtonSizes()
								end,
								min = .5,
								max = 2,
                                step = .1,
								isPercent = true
							},
							buttontype = {
								type = "text",
								name = L["Button Type"],
								desc = L["Choose whether this button casts a spell, uses a macro, or uses an item."],
								get = function() return Cryolysis2.db.profile.BottomCustomButtonType end,
								set = function(v) 
								Cryolysis2.db.profile.BottomCustomButtonType = v
								Cryolysis2:UpdateCustomButtonAttributes("bottom")
								end,
								validate = {["spell"] = L["Spell"], ["macro"] = L["Macro"], ["item"] = L["Item"]}
							},
							bottomleftclick = {
								type = "text",
								name = L["Left Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.BottomSpellLeft end,
								set = function(v) 
								Cryolysis2.db.profile.BottomSpellLeft = v
								Cryolysis2:UpdateBottomButton(1) 
								end
							},
							bottomrightclick = {
								type = "text",
								name = L["Right Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.BottomSpellRight end,
								set = function(v) 
								Cryolysis2.db.profile.BottomSpellRight = v
								Cryolysis2:UpdateBottomButton(2) 
								end
							},
							bottommidclick = {
								type = "text",
								name = L["Middle Click Spell"],
								desc = L["Type in the name of the spell that will be cast when left clicking this button."],
								usage = L["Enter a spell, macro, or item name and PRESS ENTER.  Capitalization matters."],
								get = function() return Cryolysis2.db.profile.BottomSpellMid end,
								set = function(v) 
								Cryolysis2.db.profile.BottomSpellMid = v 
								Cryolysis2:UpdateBottomButton(3) 
								end
							},
							movebottom = {
								type = "execute",
								name = L["Move Clockwise"],
								desc = L["Move this button one position clockwise."],
								func = function() Cryolysis2:IncrementButton("Cryo2BottomCustomButton") end,
							}
						}
					}
				}
			}
		}
	}
	
	-- Private variables
	Cryolysis2.Private = {

		Language = GetLocale(),
		englishFaction = UnitFactionGroup("player"),
        InCombat = false,

		-- Variables for tracking the status of the mage
		TradeWindowOpen = false,

		-- Variables for tracking whether the mage has certain spells/items
		HighestWaterId = 0,
		HighestWaterOnHand = 0,
		HighestWaterCount = 0,
		WaterName = {
			L["Conjured Water"],
			L["Conjured Fresh Water"],
			L["Conjured Purified Water"],
			L["Conjured Spring Water"],
			L["Conjured Mineral Water"],
			L["Conjured Sparkling Water"],
			L["Conjured Crystal Water"],
			L["Conjured Mountain Spring Water"],
			L["Conjured Glacier Water"],
			L["Conjured Manna Biscuit"]
		}, -- Item IDs for Mage conjured water, in order from lowest amount of mana restored to highest.  ORDER DOES MATTER
					
		WaterTex = {
			"INV_Drink_06",
			"INV_Drink_07",
			"INV_Drink_Milk_02",
			"INV_Drink_10",
			"INV_Drink_09",
			"INV_Drink_11",
			"INV_Drink_18",
			"INV_Drink_Waterskin_11",
			"INV_Drink_16",
			"INV_Misc_Food_100"
		}, -- Textures for the water button

		HighestFoodId = 0,
		HighestFoodOnHand = 0,
		HighestFoodCount = 0,
		FoodName = {
			L["Conjured Muffin"],
			L["Conjured Bread"],
			L["Conjured Rye"],
			L["Conjured Pumpernickel"],
			L["Conjured Sourdough"],
			L["Conjured Sweet Roll"],
			L["Conjured Cinnamon Roll"],
			L["Conjured Croissant"],
			L["Conjured Manna Biscuit"]
		}, -- Item IDs for Mage conjured bread, in order from lowest amount of health restored to highest.  ORDER DOES MATTER
					
		FoodTex = {
			"INV_Misc_Food_10",
			"INV_Misc_Food_09",
			"INV_Misc_Food_12",
			"INV_Misc_Food_08",
			"INV_Misc_Food_11",
			"INV_Misc_Food_33",
			"INV_Misc_Food_73CinnamonRoll",
			"INV_Misc_Food_32",
			"INV_Misc_Food_100"
		},	-- Textures for the food button
		HighestManaGemId = 0,
		HighestManaGemOnHand = 0,
		NextConjurableManaGem = 0,
		ManaGemCooldownBool = false,
		ManaGemTex = {
			"INV_Misc_Gem_Emerald_01",
			"INV_Misc_Gem_Emerald_02",
			"INV_Misc_Gem_Opal_01",
			"INV_Misc_Gem_Ruby_01",
			"INV_Misc_Gem_Stone_01"
		},
		ManaGemName = {
			L["Mana Agate"],
			L["Mana Jade"],
			L["Mana Citrine"],
			L["Mana Ruby"],
			L["Mana Emerald"]
		},
		ConjureGemName = {
			BS["Conjure Mana Agate"],
			BS["Conjure Mana Jade"],
			BS["Conjure Mana Citrine"],
			BS["Conjure Mana Ruby"],
			BS["Conjure Mana Emerald"]
		},
		HasManaGem = {
			false,
			false,
			false,
			false,
			false
		},  -- Whether the user has a manastone conjured.  Ordered lowest to highest.
		CanConjureGem = {
			false,
			false,
			false,
			false,
			false
		},  -- Whether the user has the ability to conjure each rank of mana gem.  Ordered lowest to highest.
		GemMenu = {
			"Cryo2GemMenu1",
			"Cryo2GemMenu2",
			"Cryo2GemMenu3",
			"Cryo2GemMenu4",
			"Cryo2GemMenu5"
		},
		
		HasEvocation = false,		-- Has the mage learned Evocation yet?
		HasArcaneIntellect = false,	-- Has the mage learned AI yet?
		HasFrostArmor = false,		-- Has the mage learned Frost Armor yet?
		HasIceArmor = false,		-- Has the mage learned Ice Armor yet?
		HasMoltenArmor = false,		-- Has the mage learned Molten Armor yet?
		HasMageArmor = false,		-- Has the mage learned Mage Armor yet?
		HasFireWard = false,		-- Has the mage learned Fire Ward yet?
		HasFrostWard = false,		-- Has the mage learned Frost Ward yet?
		HasSlowFall = false,		-- Has the mage learned Slow Fall yet?
		HasRemoveCurse = false,		-- Has the mage learned Remove Lesser Curse yet?
		HasManaShield = false,		-- Has the mage learned Mana Shield yet?
		HasIceBarrier = false,		-- Has the mage learned Ice Barrier? (talent based)
		HasDampenMagic = false,		-- Has the mage learned Dampen Magic yet?
		HasAmplifyMagic = false,	-- Has the mage learned Amplify Magic yet?
		HasArcaneBrilliance = false,	-- Has the mage learned Arcane Brilliance yet?
		HasRitual = false,		-- Has the mage learned Ritual of Refreshment yet?
		
		CanConjureFood = false,		-- Used to detect what buttons to show for lower level characters
		CanConjureWater = false,	-- Used to detect what buttons to show for lower level characters
		
		--[[
		Spell Menu Info:
			1 = Armor
			2 = AI/AB
			3 = Amplify/Dampen
			4 = Ice Barrier/Mana Shield
			5 = Fire/Frost Ward
			6 = Remove Lesser Curse
			7 = Slow Fall
		]]
		HasSpellMenuBuff = {
			false,
			false,
			false,
			false,
			false,
			false,
			false
		},
		BuffMenu = {
			"Cryo2BuffMenu1",
			"Cryo2BuffMenu2",
			"Cryo2BuffMenu3",
			"Cryo2BuffMenu4",
			"Cryo2BuffMenu5",
			"Cryo2BuffMenu6",
			"Cryo2BuffMenu7"
		},
		BuffMenuVisible = false,
		
		--[[
		Teleports/Portals: 
			1 = IF/Org
			2 = SW/UC
			3 = Darn/TB
			4 = Exodar/Silvermoon
			5 = Shat
		]]
		AllianceLocs = {
			BZ["Ironforge"],
			BZ["Stormwind City"],
			BZ["Darnassus"],
			BZ["The Exodar"],
			BZ["Shattrath City"],
		},
		HordeLocs = {
			BZ["Orgrimmar"],
			BZ["Undercity"],
			BZ["Thunder Bluff"],
			BZ["Silvermoon City"],
			BZ["Shattrath City"],
		},
		HasPortMenuPort = {
			false,
			false,
			false,
			false,
			false
		},
		HasPortMenuTeleport = {
			false,
			false,
			false,
			false,
			false
		},
		HasATelePort = false,
		HasAPort = false,
		PortMenu = {
			"Cryo2PortMenu1",
			"Cryo2PortMenu2",
			"Cryo2PortMenu3",
			"Cryo2PortMenu4",
			"Cryo2PortMenu5"
		},
		PortMenuVisible = false,

		-- Cooldown variables
		EvocationCooldown = 0,
		EvocationCooldownText = "",
		ManastoneCooldown = 0,
		ManastoneCooldownText = "",
		ColdsnapCooldown = 0,
		ColdsnapCooldownText = "",
		IceblockCooldown = 0,
		IceblockCooldownText = "",
		
		-- Mount variables
        Mounts = {},
        LeftMountIcon = nil,
        RightMountIcon = nil,
        HasMount = false,
		MountTex = {
			"INV_Misc_Horn_01",
			"Ability_Mount_MountainRam",
			"Ability_Mount_Raptor",
			"Ability_Mount_JungleTiger",
			"Ability_Mount_Undeadhorse",
			"Ability_Mount_MechaStrider",
			"Ability_Mount_RidingHorse",
			"Ability_Mount_Kodo_03",
			"Ability_Mount_NightmareHorse",
			"Ability_Mount_Kodo_01",
			"Ability_Mount_Kodo_02",
			"Ability_Mount_WhiteDireWolf",
			"Ability_Mount_BlackDireWolf",
			"Ability_Mount_BlackPanther",
			"Ability_Mount_WhiteTiger",
			"Ability_Mount_PinkTiger",
			"INV_Misc_QirajiCrystal_05",
			"INV_Misc_Foot_Centaur",
			"Ability_Mount_RidingElekk",
			"Ability_Mount_RidingElekk_Grey",
			"Ability_Mount_RidingElekk_Purple",
			"Ability_Mount_RidingElekkElite_Blue",
			"Ability_Mount_RidingElekkElite_Green",
			"Ability_Mount_RidingElekkElite_Purple",
			"Ability_Mount_CockatriceMount_Black",
			"Ability_Mount_CockatriceMount_Blue",
			"Ability_Mount_CockatriceMount_Purple",
			"Ability_Mount_CockatriceMount",
			"Ability_Mount_CockatriceMountElite_Green",
			"Ability_Mount_CockatriceMountElite",
			"Ability_Mount_CockatriceMountElite_Purple",
			"Ability_Mount_Dreadsteed",
			"INV_Staff_08",
			"Ability_Druid_ChallangingRoar",
			"Ability_Mount_CockatriceMount_Green",
            "INV-Mount_Raven_54",
            "Ability_Mount_Gryphon_01",
			"Ability_Mount_Wyvern_01",
			"Ability_Mount_NetherdrakePurple",
			"Ability_Mount_NetherDrakeElite",
			"Ability_Hunter_Pet_NetherRay",
			"INV_Staff_08",
			"Ability_Mount_Gyrocoptor",
			"Ability_Mount_GyrocoptorElite",
			"Ability_Mount_WarHippogryph",
			"Inv_Misc_SummerFest_BrazierOrange",
		},
		
		PolyTarget,
		RandMess = {
            ["turtle"] = "",
            ["pig"] = "",
            ["sheep"] = "",
            ["ritual"] = "",
            ["teleport"] = "",
            ["portal"] = "",
        },
        RandMessage,
		
		LastSpellCast,
		LastSpellResisted,
		SpellTimers = {},
        DebuffTimers = {},
	};

	-- Setup for the reagent restocking dialog
	StaticPopupDialogs["RESTOCK_REAGENTS"] = {
		text = L["Restock all reagents?"],
		button1 = L["Yes"],
		button2 = L["No"],
		OnAccept = function() Cryolysis2:RestockReagents(); end,
		timeout = 0,
		whileDead = 0,
		hideOnEscape = 1
	}
	
	-- Setup for the class warning dialog
	StaticPopupDialogs["CLASS_WARNING"] = {
		text = L["Cryolysis 2 has detected that you are not playing a mage.  The mod has not fully loaded, but is still taking up memory.  For the best game performance, you should disable Cryolysis 2 for this character on your addon selection screen.  This message will only appear once."],
		button1 = L["Okay"],
		OnAccept = function() Cryolysis2.db.profile.ClassWarn = false; end,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	}
end